﻿Imports System.Windows.Forms
Imports System.IO
Imports System.Xml
Imports System.ServiceModel

Public Class FileStreamer
	Inherits System.Windows.Forms.Form

	Private fileInfo As FileInfo
	Private chunkSize As Integer = 50 * 1024 '50KB
	Private fileLength As Integer = 0
	Private binaryClient As Object
	Private binaryData As EntityBinary.BinaryData
	Private docAttachmentClient As Object
	Private docAttachmentData As EntityDocumentAttachment.DocumentAttachmentData
	Private output As XmlTextWriter
	Private xmlDoc As XmlDocument
	Private node As XmlNode

	Public Sub New()
		' This call is required by the Windows Form Designer.
		InitializeComponent()
		FileTextBox.Text = ""
		txtFileName.ReadOnly = True
	End Sub

	Private Function FormValidation(ByVal operation As String, Optional ByVal IsUploadButtonClicked As Boolean = False) As Boolean
		Dim filesIsSelected As Boolean = (OpenFileDialog1.FileNames.Count > 0)

		'Get Document ID (GUID)
		If IsUploadButtonClicked Then
			If String.IsNullOrEmpty(DocumentIDTextbox.Text) Then
				MsgBox("Mandatory: Document ID")
				Return False
			End If
		End If

		If operation = "Update" Then
			If String.IsNullOrEmpty(DocAttachmentID.Text) Then
				MsgBox("Mandatory: Document Attachment ID")
				Return False
			End If
		End If

		'Get File Info
		If Not filesIsSelected Then
			MsgBox("No upload file selected.")
			Return False
		End If

		Return True
	End Function

	'Private Sub UploadButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UploadButton.Click
	'	If Not FormValidation(IsUploadButtonClicked:=True) Then Exit Sub
	'	If UploadAttachment() Then MsgBox("Attachment(s) successfully uploaded.")
	'End Sub

	Private Function UploadAttachment(ByVal operation As String) As Boolean
		If FormValidation(operation) AndAlso UploadOperation(operation) Then Return True
	End Function

	Private Function ValidateOperation() As Boolean
		Try
			Dim docAttachmentData As New EntityDocumentAttachment.DocumentAttachmentData
			If Not String.IsNullOrEmpty(DocAttachmentID.Text) Then
				docAttachmentData.ID = New EntityDocumentAttachment.EntityGuid With {.Value = New Guid(DocAttachmentID.Text), .IsDirty = True}
			Else
				docAttachmentData.ID = New EntityDocumentAttachment.EntityGuid With {.IsDirty = True, .IsNothing = True}
			End If
			If Not String.IsNullOrEmpty(DocumentIDTextbox.Text) Then
				docAttachmentData.DocumentID = New EntityDocumentAttachment.EntityGuid With {.Value = New Guid(DocumentIDTextbox.Text), .IsDirty = True}
			Else
				docAttachmentData.DocumentID = New EntityDocumentAttachment.EntityGuid With {.IsDirty = True, .IsNothing = True}
			End If
			If Not String.IsNullOrEmpty(txtFileName.Text) Then
				docAttachmentData.AttachmentFileName = New EntityDocumentAttachment.EntityString With {.Value = txtFileName.Text, .IsDirty = True}
			Else
				docAttachmentData.AttachmentFileName = New EntityDocumentAttachment.EntityString With {.IsDirty = True, .IsNothing = True}
			End If
			docAttachmentData = docAttachmentClient.Validate(docAttachmentData)
			Return True
		Catch fe As FaultException
			ShowError(fe)
		Catch ex As Exception
			LogText("", ex.Message, , False)
		End Try
	End Function

	Private Function DeleteOperation() As Boolean
		Try
			Dim docAttachmentData As New EntityDocumentAttachment.DocumentAttachmentData
			With docAttachmentData
				.ID = New EntityDocumentAttachment.EntityGuid With {.Value = New Guid(DocAttachmentID.Text), .IsDirty = True}
			End With
			docAttachmentClient.Delete(docAttachmentData)
			Return True
		Catch fe As FaultException
			ShowError(fe)
		Catch ex As Exception
			LogText("", ex.Message, , False)
		End Try
	End Function

	Private Function RetrieveOperation() As Boolean
		Try
			Dim docAttachmentData As New EntityDocumentAttachment.DocumentAttachmentData
			With docAttachmentData
				.ID = New EntityDocumentAttachment.EntityGuid With {.Value = New Guid(DocAttachmentID.Text), .IsDirty = True}
			End With
			docAttachmentData = docAttachmentClient.Retrieve(docAttachmentData)
			LogText("Info", "Attachment Filename : " & docAttachmentData.AttachmentFileName.Value, True, True)
			LogText("Info", "Attachment File Extension : " & docAttachmentData.AttachmentFileExtension.Value, True, True)
			LogText("Info", "Document ID : " & docAttachmentData.DocumentID.Value.ToString, True, True)
			LogText("Info", "Version Number : " & docAttachmentData.VersionNumber.Value, True, True)
			Return True
		Catch fe As FaultException
			ShowError(fe)
		Catch ex As Exception
			LogText("", ex.Message, , False)
		End Try
	End Function

	Private Function UploadOperation(ByVal operation As String) As Boolean

		If OpenFileDialog1.FileNames.Count > 0 Then
			InitMetadata()
			Cursor = Cursors.WaitCursor
			LogDataView.Rows.Clear()

			Try
				progressBar1.Visible = True
				Dim countLog As Integer = 0

				LogText("Start Processing", "Upload attachment(s) chunks data", True, True)

				For Each selectedFilenamePath As String In OpenFileDialog1.FileNames
					'Show start message 
					fileInfo = New FileInfo(selectedFilenamePath)
					LogText("Info", "Filename: " & fileInfo.Name, True)

					Dim fileSizeInBytes As Long = fileInfo.Length
					LogText("", "Size: " & (Math.Round(fileSizeInBytes / 1024, 2)).ToString() & " Kb")

					'Read File
					Using iStream As New FileStream(selectedFilenamePath, FileMode.Open, FileAccess.Read, FileShare.Read)
						'###01### Start Upload: Create message header
						LogText("Start", "Create message header", True)

						'############################################################################
						'Start upload
						binaryData = New EntityBinary.BinaryData
						binaryData = binaryClient.Create(binaryData)

						LogText("End", "Create message header", True)
						'############################################################################

						'###02### Upload: Create data (chunk-by-chunk)
						LogText("Start", "Create chunk packet", True)

						'Get total size for last packet
						Dim lastPacketDataSize As Long = fileSizeInBytes Mod chunkSize

						'- How many packet we need to store all the chunk.
						'- Each packet only can stored 3800 bytes of data.
						Dim totalChunkPackets As Integer = Int((fileSizeInBytes / chunkSize)) + IIf(lastPacketDataSize > 0, 1, 0)

						'Each complete packet consists of 3800 bytes data
						Dim completePacketData() As Byte = Nothing
						completePacketData = New Byte(chunkSize - 1) {}

						Dim countPacket As Integer = 0
						Dim chunkSequenceNumber As Integer
						Dim chunkPacketToWrite() As Byte

						Dim dataToRead As Long = iStream.Length
						progressBar1.Maximum = dataToRead
						progressBar1.Visible = True

						While dataToRead > 0

							If countPacket = totalChunkPackets - 1 Then
								'1) Special handle for last packet of data due to the data size could less than 3800 bytes. 
								'   We do not need complete 3800 bytes as packet size.
								'2) Another reason is due to "File corrupted" when re-open the attachment using office application.
								'   This is because of extra data has been write into attachment as well.
								Dim lastPacketData() As Byte = Nothing
								lastPacketData = New Byte(lastPacketDataSize - 1) {}
								fileLength = iStream.Read(lastPacketData, 0, lastPacketDataSize)
								chunkPacketToWrite = lastPacketData
							Else
								'Read chunk data and assign as [complete packet]
								fileLength = iStream.Read(completePacketData, 0, chunkSize)
								chunkPacketToWrite = completePacketData
							End If

							'############################################################################
							'Upload chunk
							With binaryData
								.Sequence = New EntityBinary.EntityInteger With {.Value = countPacket, .IsDirty = True}
								.Data = New EntityBinary.EntityObject With {.Value = chunkPacketToWrite, .IsDirty = True}
							End With
							binaryData = binaryClient.Save(binaryData)
							chunkSequenceNumber = binaryData.Sequence.Value

							If Not String.IsNullOrEmpty(chunkSequenceNumber) Then
								dataToRead = dataToRead - fileLength

								'Count total packet
								countPacket += 1
								LogText("Data", "Packet [" & chunkSequenceNumber & "] uploaded.", , True)
							Else
								'Count total packet
								countPacket -= 1
								LogText("Data", "Packet [" & chunkSequenceNumber & "] failed to upload.", , False)
							End If
							'############################################################################

							'Update progress bar status
							progressBar1.Increment(fileLength)
							If progressBar1.Value = progressBar1.Maximum Then
								progressBar1.Maximum = 0
								progressBar1.Visible = False
							End If

							Application.DoEvents()
							LogDataView.Invalidate()
						End While

						iStream.Close()
						LogText("End", "Create chunk packet (" & "Total packet: " & countPacket & ")", True)

						'###03### End Upload: Combine chunk
						Try
							LogText("End Upload", "Combine chunk message", True)
							'############################################################################
							'End upload
							Dim docAttachmentData As New EntityDocumentAttachment.DocumentAttachmentData
							With docAttachmentData
								If Len(DocAttachmentID.Text) > 0 Then
									.ID = New EntityDocumentAttachment.EntityGuid With {.Value = New Guid(DocAttachmentID.Text), .IsDirty = True}
								Else
									'.ID = New EntityDocumentAttachment.EntityGuid With {.IsNothing = True}
								End If
								.AttachmentFileName = New EntityDocumentAttachment.EntityString With {.Value = Path.GetFileName(fileInfo.FullName), .IsDirty = True}
								.AttachmentFileExtension = New EntityDocumentAttachment.EntityString With {.Value = fileInfo.Extension, .IsDirty = True}
								.DocumentID = New EntityDocumentAttachment.EntityGuid With {.Value = New Guid(DocumentIDTextbox.Text), .IsDirty = True}
								.MessageID = New EntityDocumentAttachment.EntityGuid With {.Value = binaryData.MessageID.Value, .IsDirty = True}
							End With
							Select Case operation
								Case "Create"
									docAttachmentData = docAttachmentClient.Create(docAttachmentData)
								Case "Save"
									docAttachmentData = docAttachmentClient.Save(docAttachmentData)
								Case "Update"
									docAttachmentClient.Update(docAttachmentData)
							End Select

							txtFileName.Text = docAttachmentData.AttachmentFileName.Value
							If Not docAttachmentData.ID.Value.Equals(System.Guid.Empty) Then
								DocAttachmentID.Text = docAttachmentData.ID.Value.ToString
							End If

						Catch fe As FaultException
							ShowError(fe)
							Return False
						Catch ex As Exception
							LogText("", ex.Message, , False)
							Return False
						End Try

					End Using

					Application.DoEvents()
				Next

				LogText("End Processing", "Upload attachment(s) chunks data", True, True)

			Catch ex As Exception
				LogText("Error", ex.Message, True, False)
				Return False
			Finally
				Cursor = Cursors.Default
			End Try
			Cursor = Cursors.Default
			Return True
		Else
			Return False 'No attachment upload
		End If
	End Function

	'Add data log using dataview control
	Private Sub LogText(ByVal Action As String, ByVal Description As String, Optional ByVal IsNewAction As Boolean = False, Optional ByVal IsSuccess As Boolean = True)
		With LogDataView
			.Rows.Add()
			If IsNewAction Then
				'Color the column
				If Action = "Start Processing" OrElse Action = "End Processing" Then
					.Item("ActionColumn", .Rows.Count - 1).Style.BackColor = Color.Green
					If Action = "End Processing" Then LogText("", "")

				ElseIf Action = "Status" Then
					.Item("ActionColumn", .Rows.Count - 1).Style.BackColor = Color.DarkOrange

				ElseIf Action = "Info" Then
					.Item("ActionColumn", .Rows.Count - 1).Style.BackColor = Color.Honeydew

				ElseIf Action = "Start" OrElse Action = "End" Then
					.Item("ActionColumn", .Rows.Count - 1).Style.BackColor = Color.GreenYellow
					If Action = "End" Then LogText("", "")
				Else
					.Item("ActionColumn", .Rows.Count - 1).Style.BackColor = Color.MediumVioletRed
				End If

				'Bold the text
				.Item("ActionColumn", .Rows.Count - 1).Style.Font = New Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold)
			End If

			.Item("DateColumn", .Rows.Count - 1).Value = Now
			.Item("ActionColumn", .Rows.Count - 1).Value = Action
			.Item("DescriptionColumn", .Rows.Count - 1).Value = Description

			'.Item("StatusImageColumn", .Rows.Count - 1).Value = IIf(IsSuccess, My.Resources.MyResources.check, My.Resources.MyResources.cross)
			'.Item("StatusColumn", .Rows.Count - 1).Value = IIf(IsSuccess, "Process Success", "Process Fail")

			.FirstDisplayedScrollingRowIndex = .Rows.Count - 1
		End With
	End Sub

	Private Sub ClearButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ClearButton.Click
		LogDataView.Rows.Clear()
	End Sub

	Private Sub ExportButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExportButton.Click
		Try
			Dim currentDateTime As DateTime = DirectCast(Now, DateTime)
			SaveDataLogFileDialog.FileName = "PMB-01-500kb-DataLog.xml"	'currentDateTime.ToString("yyyy-MM-ddTHHmmss") & "-DataLog.xml"
			If Not String.IsNullOrEmpty(SaveDataLogFileDialog.FileName) Then
				ExportLogAsXmlFile(SaveDataLogFileDialog.FileName)
			End If
		Catch ex As Exception
			MessageBox.Show("Unable create data log file", "Error", MessageBoxButtons.OK)
		End Try
	End Sub

	Private Sub ExportLogAsXmlFile(ByVal SaveFilename As String)
		If SaveDataLogFileDialog.ShowDialog = Windows.Forms.DialogResult.OK Then
			'Remove file before create new data log file
			If File.Exists(SaveFilename) Then File.Delete(SaveFilename)
			If File.Exists(SaveDataLogFileDialog.FileName) Then File.Delete(SaveDataLogFileDialog.FileName)

			'Stream to output file
			output = New XmlTextWriter(SaveDataLogFileDialog.FileName, System.Text.Encoding.UTF8)
			With output
				.Formatting = Formatting.Indented
				.Indentation = 4
				.WriteStartDocument()

				.WriteStartElement("DataLogs") 'Start: <DataLogs>
				For rowCount As Integer = 0 To LogDataView.Rows.Count - 1
					.WriteStartElement("DataLog")	'Start: <DataLog>

					.WriteStartElement("Date")
					.WriteCData(LogDataView.Item("DateColumn", rowCount).Value)
					.WriteEndElement()

					.WriteStartElement("Action")
					.WriteCData(LogDataView.Item("ActionColumn", rowCount).Value)
					.WriteEndElement()

					.WriteStartElement("Description")
					.WriteCData(LogDataView.Item("DescriptionColumn", rowCount).Value)
					.WriteEndElement()

					.WriteEndElement() 'End: <DataLog>
				Next
				.WriteEndElement() 'End: <DataLogs>

				.WriteEndDocument()
				.Close()
			End With
		End If
	End Sub

	''' <summary>
	''' Read all the content from a file as byte array
	''' </summary>
	''' <param name="strFilePath"></param>
	''' <returns></returns>
	''' <remarks></remarks>
	Public Shared Function ReadFileAsBytes(ByVal strFilePath As String) As Byte()

		Dim fs As System.IO.FileStream = New System.IO.FileStream(strFilePath, System.IO.FileMode.Open, System.IO.FileAccess.Read, System.IO.FileShare.ReadWrite)
		Dim br As System.IO.BinaryReader = New System.IO.BinaryReader(fs)
		Dim baResult() As Byte = Nothing
		Try
			baResult = New Byte(fs.Length - 1) {}
			br.Read(baResult, 0, baResult.Length)
		Finally
			br.Close()
			fs.Close()
		End Try
		Return baResult
	End Function

	''' <summary>
	''' Read all the content from a file as string in default encoding
	''' </summary>
	''' <param name="strFilePath"></param>
	''' <returns>dest string in default encoding</returns>
	''' <remarks></remarks>
	Public Shared Function ReadFileAsString(ByVal strFilePath As String) As String
		'Dim encDefault As System.Text.Encoding = System.Text.Encoding.GetEncoding(0)
		Dim encDefault As System.Text.Encoding = System.Text.Encoding.UTF8
		Dim fs As System.IO.FileStream = New System.IO.FileStream(strFilePath, System.IO.FileMode.Open, System.IO.FileAccess.Read, System.IO.FileShare.ReadWrite)
		Dim br As System.IO.BinaryReader = New System.IO.BinaryReader(fs)
		Dim strResult As String = Nothing
		Try
			Dim bData() As Byte = New Byte(fs.Length) {}
			br.Read(bData, 0, bData.Length)
			strResult = encDefault.GetString(bData)
		Finally
			br.Close()
			fs.Close()
		End Try
		Return strResult

	End Function

	Private Sub BrowseButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BrowseButton.Click
		FileTextBox.Text = ""

		If OpenFileDialog1.ShowDialog = Windows.Forms.DialogResult.OK Then
			If OpenFileDialog1.FileNames.Count > 1 Then
				For Each selectedFilenamePath As String In OpenFileDialog1.FileNames
					FileTextBox.Text = FileTextBox.Text & selectedFilenamePath & vbCrLf
				Next
			Else
				FileTextBox.Text = OpenFileDialog1.FileName
			End If			
		End If
	End Sub

	Private Sub btnDownload_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDownload.Click
		Try
			Dim docAttachmentData As New EntityDocumentAttachment.DocumentAttachmentData
			With docAttachmentData
				.ID = New EntityDocumentAttachment.EntityGuid With {.Value = New Guid(DocAttachmentID.Text), .IsDirty = True}
				''Set download mode
				.PrepareDownload = New EntityDocumentAttachment.EntityBoolean With {.Value = True, .IsDirty = True}
			End With
			docAttachmentData = docAttachmentClient.Retrieve(docAttachmentData)

			'download path
			DownloadPath.Text = "C:\" & docAttachmentData.AttachmentFileName.Value

			Dim fs As New FileStream(DownloadPath.Text, FileMode.OpenOrCreate, FileAccess.Write)
			Dim writer As BinaryWriter = New BinaryWriter(fs)

			Dim binaryData As New EntityBinary.BinaryData
			binaryData.MessageID = New EntityBinary.EntityGuid With {.Value = docAttachmentData.MessageID.Value, .IsDirty = True}
			

			Dim sequence As Integer = 0
			binaryData.Sequence = New EntityBinary.EntityInteger With {.Value = sequence, .IsDirty = True}
			binaryData.Encoded = New EntityBinary.EntityBoolean With {.Value = True, .IsDirty = True}
			binaryData = binaryClient.Retrieve(binaryData)

			Dim base64String As String = binaryData.DataString.Value
			'Dim chunk As Byte()
			'chunk = binaryData.Data.Value
			'While Not IsNothing(chunk)
			'	writer.Write(chunk, 0, chunk.Length)
			'	writer.Flush()
			'	chunk = Nothing
			'	sequence = sequence + 1
			'	binaryData.Sequence = New EntityBinary.EntityInteger With {.Value = sequence, .IsDirty = True}
			'	binaryData = binaryClient.Retrieve(binaryData)
			'	chunk = binaryData.Data.Value
			'	If IsNothing(binaryData.Data.Value) Then Exit While
			'End While

			'writer.Close()
			'fs.Close()

			'LogDataView.Rows.Clear()
			'LogText("Info", "Attachment Filename : " & docAttachmentData.AttachmentFileName.Value, True, True)
			'LogText("Info", "Attachment File Extension : " & docAttachmentData.AttachmentFileExtension.Value, True, True)
			'LogText("Info", "Document ID : " & docAttachmentData.DocumentID.Value.ToString, True, True)
			'LogText("Info", "Version Number : " & docAttachmentData.VersionNumber.Value, True, True)
			'LogText("Status", "Attachment successfully downloaded", True, True)

		Catch fe As FaultException
			ShowError(fe)
		Catch ex As Exception
			MessageBox.Show(ex.Message)
		End Try
	End Sub

	Private Sub ShowError(ByVal exc As Object)
		Dim fe As Object
		Select Case True
			Case InStr(exc.GetType.ToString, "EntityBinary")
				fe = DirectCast(exc, FaultException(Of EntityBinary.EntityFault))
			Case InStr(exc.GetType.ToString, "EntityDocumentAttachment")
				fe = DirectCast(exc, FaultException(Of EntityDocumentAttachment.EntityFault))
		End Select

		Dim sb As New System.Text.StringBuilder
		sb.AppendLine("An error has occured!")
		sb.AppendLine("")
		sb.AppendFormat("{0}: [{1}]" & vbCrLf, "Exception Type", fe.Code.Name)
		sb.AppendFormat("{0}: [{1}]" & vbCrLf, "Exception Message", fe.Message)
		sb.AppendLine("")

		If Not IsNothing(fe.Detail.Exceptions) AndAlso UBound(fe.Detail.Exceptions) > -1 Then
			For Each info In fe.Detail.Exceptions
				With sb
					.AppendFormat("{0}: [{1}]" & vbCrLf, "Property", info.PropertyName)
					.AppendFormat("{0}: [{1}]" & vbCrLf, "Message", info.Message)
					.AppendLine("")
				End With
			Next
		End If

		MessageBox.Show(sb.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
	End Sub

	Private Sub InitMetadata()
		' Set client impersonation
		If IsNothing(binaryClient) Then
			binaryClient = New EntityBinary.BinaryClient
			If Not String.IsNullOrEmpty(txtUsername.Text) Then
				binaryClient.ChannelFactory.Credentials.Windows.ClientCredential = New System.Net.NetworkCredential(txtUsername.Text, txtPassword.Text)
			End If
			binaryClient.ClientCredentials.Windows.AllowedImpersonationLevel = System.Security.Principal.TokenImpersonationLevel.Impersonation
			If txtAddress.Text.Length > 0 Then
				binaryClient.Endpoint.Address = New System.ServiceModel.EndpointAddress(txtAddress.Text & "/services/exact.entity.binary.svc")
			End If
		End If

		If IsNothing(docAttachmentClient) Then
			docAttachmentClient = New EntityDocumentAttachment.DocumentAttachmentClient
			If Not String.IsNullOrEmpty(txtUsername.Text) Then
				docAttachmentClient.ChannelFactory.Credentials.Windows.ClientCredential = New System.Net.NetworkCredential(txtUsername.Text, txtPassword.Text)
			End If
			docAttachmentClient.ClientCredentials.Windows.AllowedImpersonationLevel = System.Security.Principal.TokenImpersonationLevel.Impersonation
			If txtAddress.Text.Length > 0 Then
				docAttachmentClient.Endpoint.Address = New System.ServiceModel.EndpointAddress(txtAddress.Text & "/services/exact.entity.documentattachment.svc")
			End If
		End If
	End Sub

	Private Sub btnGo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGo.Click
		InitMetadata()
	End Sub

	Private Sub btnClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClearDocID.Click
		DocumentIDTextbox.Text = ""
	End Sub

	Private Sub btnClearAttachID_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClearAttachID.Click
		DocAttachmentID.Text = ""
	End Sub

	Private Sub btnCreate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCreate.Click
		If Not FormValidation("Create", IsUploadButtonClicked:=True) Then Exit Sub
		If UploadAttachment("Create") Then LogText("Status", "Attachment created successfully", True, True)
	End Sub

	Private Sub btnUpdate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnUpdate.Click
		If Not FormValidation("Update", IsUploadButtonClicked:=True) Then Exit Sub
		If UploadAttachment("Update") Then LogText("Status", "Attachment updated successfully", True, True)
	End Sub

	Private Sub btnSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSave.Click
		If Not FormValidation("Save", IsUploadButtonClicked:=True) Then Exit Sub
		If UploadAttachment("Save") Then LogText("Status", "Attachment saved successfully", True, True)
	End Sub

	Private Sub btnDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDelete.Click
		LogDataView.Rows.Clear()
		If Not String.IsNullOrEmpty(DocAttachmentID.Text) Then
			InitMetadata()
			If DeleteOperation() Then
				LogText("Status", "Attachment deleted successfully", True, True)
			End If
		Else
			MsgBox("Mandatory: Document Attachment ID")
		End If
	End Sub

	Private Sub btnRetrieve_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRetrieve.Click
		LogDataView.Rows.Clear()
		If Not String.IsNullOrEmpty(DocAttachmentID.Text) Then
			InitMetadata()
			If RetrieveOperation() Then
				LogText("Status", "Attachment information retrieved successfully", True, True)
			End If
		Else
			MsgBox("Mandatory: Document Attachment ID")
		End If
	End Sub

	Private Sub btnValidate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnValidate.Click
		LogDataView.Rows.Clear()
		InitMetadata()
		If ValidateOperation() Then
			LogText("Status", "Attachment validated successfully", True, True)
		End If
	End Sub
End Class

